import sys
class Logger():

    def __init__(self, log, quiet, verbose, echo_limit, log_file_location):
        self.log_flag = log
        self.quiet_flag = quiet
        self.verbose_flag = verbose
        self.log_file_location = log_file_location
        self.echo_limit = echo_limit

    def start(self):
        if self.log_flag:
            self.log_file = open(self.log_file_location, 'w')

    def log(self, message, important=False):
        message = self.truncate(str(message), self.echo_limit)
        if important or self.verbose_flag and not self.quiet_flag:
            print(message)
        if self.log_flag:
            self.log_file.write("\n"+message)

    def close(self):
        if self.log_flag:
            self.log_file.close()    

    def truncate(self, string, length):
        try:
            if len(string) <= length or length == 0:
                return string
            else:
                return string[:length]
        except TypeError as err:
            self.log("Tried to truncate a non-string", True)
            return string



from bs4 import BeautifulSoup
import requests
import webbrowser
import os

def get_soup(url):
    web_page = requests.get(url)
    return BeautifulSoup(web_page.text, "html.parser")

def get_link(soup, locator, url, logger):
    path = locator.split(":")
    working_area = soup
    for i, node in enumerate(path):
        try:
            if i > 0:
                logger.log("Current Scope:")
                logger.log(str(working_area))
        
            if node.startswith("."):
                logger.log("\n# Finding tag with class: " + node[1:])
                working_area = working_area.find(class_=node[1:])
            elif node.startswith("#"):
                logger.log("\n# Finding tag with class: " + node[1:])
                working_area = working_area.find(id=node[1:])
            elif node.startswith("^"):
                logger.log("\n# Moving to parent tag...")
                working_area = soup.find(working_area).parent
            elif node.startswith("*"):
                logger.log("\n# Getting attribute: href")
                working_area = working_area.get_attribute_list("href")[0]
            elif node.startswith("@"):
                logger.log("\n# Getting attribute: " + node[1:])
                working_area = working_area.get_attribute_list(node[1:])
            else:
                logger.log("\n# Finding tag: " + node + "...")
                working_area = working_area.find(name=node)
        except AttributeError:
            logger.log("\nUnable to locate link.  Please correct config file.", True)
        
    try:
        logger.log("# Current Location: ")
        logger.log(working_area)
        if not working_area.startswith("http"):
            logger.log("# Relative URL found.")
            working_area = url[:find_nth(url, "/", 3)] + working_area
            logger.log("# Edited location: " + working_area)
    except Exception:
        logger.log("# Empty Location")
        return False

    
    return working_area

def download(out_folder_location, file_name, url, locator, folder, logger):
    logger.log("\n\n######## " + file_name + " ########", True)
    logger.log("Searching for download at URL: " + url, True)

    download_link = False
    if locator == "+":
        logger.log("# Downloading directly at URL: " + url, True)
        download_link = url
    elif locator == "-":
        logger.log("# Manual download required.  Opening page in default browser...", True)
        webbrowser.open(url)
        status = "Manual"
    else:
        soup = get_soup(url)
        download_link = get_link(soup, locator, url, logger)
    
    if not download_link == False:
        download_file = requests.get(download_link, allow_redirects = True)
        out_folder = os.path.join(out_folder_location, folder)
        if not os.path.exists(out_folder):
            os.makedirs(out_folder)
        
        out = os.path.join(out_folder, file_name)
        open(out, 'wb').write(download_file.content)
        logger.log("# "+file_name+" downloaded.", True)
        logger.log("# Saved to "+out, True)
        result = "Success"
    else:
        logger.log("# Unable to save file.  See log file for further details.")
        result = "Failed"
    
    return result



example_config = [
    ["Adium","https://adium.im",".downloadInfo:a:*href","dmg","mac"],
    ["CCleaner","https://www.ccleaner.com/ccleaner/download?mac","#GTM__link--CC-mac-download-footer:*href","dmg","mac"],
    ["DiskInventoryX","http://derlien.com/downloads/index.html","#osversion:^:a:*href","dmg","mac"],
    ["EtreCheckPro","https://www.etrecheck.com/details","@Download EtreCheckPro directly:*href","app","mac"],
    ["MalwareBytes","https://www.malwarebytes.com/mac-download","#cta-mac-download-download-mbmac-en:*href","pkg","mac"],
    ["Perian","https://www.perian.org",".download_wrapper:h3:a:*href","dmg","mac"],
    ["Mozilla_Firefox","https://mozilla.org/en-US/firefox/all","-","dmg","mac"],
    ["Google_Chrome","https://dl.google.com/chrome/mac/stable/CHFA/googlechrome.dmg","+","dmg","mac"],
    ["Adware_Removal_Tool_by_TSA","https://www.techsupportall.com/download-art","+","exe","windows"],
    ["Super_Anti_Spyware","http://cdn.superantispyware.com/SUPERAntiSpyware.exe","+","exe","windows"],
    ["HitmanPro","https://www.hitmanpro.com/en-us/downloads.aspx","-","exe","windows"],
    ["HitmanPro.Alert","https://www.hitmanpro.com/en-us/downloads.aspx","-","exe","windows"]
]



#core.py
#Author: Willem Hunt, whunt1@uvm.edu

import os
import sys
import csv
import argparse

from . import fetch
from . import logger
from . import config as cfg

class Core():

    def __init__(self):
        self.download_results = {}

        self.parser = argparse.ArgumentParser(description="Batch-download files from the internet", prog="Snag")
        
        loudness = self.parser.add_mutually_exclusive_group()
        loudness.add_argument("-v", "--verbose", action="store_true", help="Enable verbose mode.", default=False)
        loudness.add_argument("-q", "--quiet", action="store_true", help="Enable quiet mode.", default=False)
        self.parser.add_argument("-c", "--chars", type=int, help="Maximum characters to print to screen per log entry.", default=0)
        self.parser.add_argument("-l", "--log", help="Enable logging at specified location", default="")
        self.parser.add_argument("-o", "--out", help="Specify output directory", default=os.getcwd())
        
        config = self.parser.add_mutually_exclusive_group()
        config.add_argument("-e", "--example", help="Use example config.", action="store_true", default=False)
        config.add_argument("config", nargs="?", help="File to read download data from.", default="")
        
        args = self.parser.parse_args()

        if not args.log == "":
            self.log = True
            self.log_file = args.log
        else:
            self.log = False
            self.log_file = ""

        self.quiet          = args.quiet
        self.verbose        = args.verbose
        self.echo_limit     = args.chars
        self.config_file    = args.config
        self.out_dir        = args.out
        self.use_example    = args.example

        
        slash_pos = self.log_file.rfind("/")
        if slash_pos > -1:
            full_path = os.path.join(os.getcwd(), self.log_file)
            log_folder = full_path[:full_path.rfind("/")]
            if self.log and not os.path.exists(log_folder):
                os.makedirs(log_folder)    

        #Initialize Logger object
        self.logger = logger.Logger(self.log, self.quiet, self.verbose,self.echo_limit, self.log_file)           
        self.logger.start()

        self.logger.log("Using the following options:")
        self.logger.log("Logging: " + str(self.log))
        self.logger.log("Verbose Mode: " + str(self.verbose))
        self.logger.log("Quiet Mode: " + str(self.quiet))
        if self.echo_limit > 0:
            self.logger.log("Message Character Limit: " + self.echo_limit)

        #Read programs from config
        self.programs = self.read_config()

    def download(self, programs):

        if not self.programs:
            return False

        for program in programs:
            name    = program[0] + "." + program[3]
            url     = program[1]
            locator = program[2]
            folder  = program[4]
            success = fetch.download(self.out_dir, name, url, locator, folder, self.logger)
            print("Status: " + success)
            self.download_results[name] = success
        return True

    def read_config(self):
        programs = []
        if not self.config_file == "":
            self.logger.log("# Reading config file  at: " + self.config_file)
            try:
                with open(self.config_file) as config:
                    lines = csv.reader(config, delimiter=',')
                    for line in lines:
                        if not line[0][0] == "#":
                            programs.append(line)
            except:
                self.logger.log("Unable to find config file at: " + self.config_file, True)
                self.logger.log("Please specify config file location with \"config=\" option.")
                return False
        elif self.use_example:
            self.logger.log("# Using example config")
            programs = cfg.example_config
        else:
            self.parser.print_help()

        self.logger.log("\n\nFiles to download:")
        for program in programs:
            self.logger.log(program[0] + "." + program[3] + " - " + program[1])

        return programs
    
    def log_results(self):
        self.logger.log("\nDownload Results:", True)
        for download, result in self.download_results.items():
            self.logger.log(("{:40s} {:10s}".format(download+":", result)))


